@@ -6,6 +6,7 @@ from weconnect.addressable import AddressableLeaf
 from weconnect.elements.charging_status import ChargingStatus
 from weconnect.elements.plug_status import PlugStatus
 from weconnect.elements.control_operation import ControlOperation
+from weconnect.errors import SetterError
 
 from vwsfriend.homekit.genericAccessory import GenericAccessory
 
@@ -22,7 +23,7 @@ class Charging(GenericAccessory):
 
         self.chargingControl = chargingControl
 
-        self.service = self.add_preload_service('Outlet', ['Name', 'ConfiguredName', 'On', 'OutletInUse', 'RemainingDuration', 'Consumption'])
+        self.service = self.add_preload_service('Outlet', ['Name', 'ConfiguredName', 'On', 'OutletInUse', 'RemainingDuration', 'Consumption', 'StatusFault'])
         self.batteryService = self.add_preload_service('BatteryService', ['BatteryLevel', 'StatusLowBattery', 'ChargingState'])
         self.service.add_linked_service(self.batteryService)
 
@@ -58,6 +59,7 @@ class Charging(GenericAccessory):
             self.setChargingState(chargingStatus.chargingState)
 
         self.addNameCharacteristics()
+        self.addStatusFaultCharacteristic()
 
     def setOnState(self, chargingState):
         if self.charOn is not None:
@@ -113,12 +115,16 @@ class Charging(GenericAccessory):
 
     def __onOnChanged(self, value):
         if self.chargingControl.enabled:
-            if value:
-                LOG.debug('Switch charging on')
-                self.chargingControl.value = ControlOperation.START
-            else:
-                LOG.debug('Switch charging off')
-                self.chargingControl.value = ControlOperation.STOP
+            try:
+                if value:
+                    LOG.info('Switch charging on')
+                    self.chargingControl.value = ControlOperation.START
+                else:
+                    LOG.info('Switch charging off')
+                    self.chargingControl.value = ControlOperation.STOP
+            except SetterError as setterError:
+                LOG.error('Error starting charging: %s', setterError)
+                self.setStatusFault(1, timeout=120)
         else:
             LOG.error('Charging cannot be controled')
 
