@@ -2,6 +2,7 @@ from .battery import Battery
 from .charge import Charge
 from .charging_session import ChargingSession
 from .climatization import Climatization
+from .journey import Journey
 from .location import Location
 from .online import Online
 from .range import Range
@@ -11,3 +12,5 @@ from .vehicle_settings import VehicleSettings
 from .vehicle import Vehicle
 from .charger import Charger, Operator
 from .settings import Settings
+from .weconnect_error import WeConnectError
+from .weconnect_responsetime import WeConnectResponsetime
