@@ -7,8 +7,7 @@ from sqlalchemy import and_
 
 from flask import Blueprint, render_template, current_app, abort, request, flash, redirect, url_for, send_file
 from flask_wtf import FlaskForm
-from wtforms import SubmitField, HiddenField, StringField, SelectField, FileField
-from wtforms.fields.html5 import DateTimeField, IntegerField, DecimalField
+from wtforms import SubmitField, HiddenField, StringField, SelectField, FileField, DateTimeField, IntegerField, DecimalField
 from wtforms.validators import DataRequired, NumberRange, Optional
 
 from weconnect.errors import RetrievalError
@@ -62,7 +61,7 @@ class ChargingSessionEditForm(FlaskForm):
     disconnected = DateTimeField('Plug Disconnect Date and Time (UTC)', validators=[Optional()])
     # maxChargeCurrentACSetting
     targetSOCSetting_pct = IntegerField('Target SoC Setting', validators=[Optional(), NumberRange(min=0, max=100)])
-    maximumChargePower_kW = IntegerField('Maximum Power during session', validators=[Optional(), NumberRange(min=0)])
+    maximumChargePower_kW = DecimalField('Maximum Power during session', places=4, validators=[Optional(), NumberRange(min=0)])
     acdc = SelectField("AC/DC", choices=ACDC.choices(), coerce=ACDC.coerce, validators=[DataRequired()])
     startSOC_pct = IntegerField('SoC at start', validators=[Optional(), NumberRange(min=0, max=100)])
     endSOC_pct = IntegerField('SoC at end', validators=[Optional(), NumberRange(min=0, max=100)])
